import Vue from "vue";
import VueRouter from "vue-router";
import axios from "axios";
import { localize } from "vee-validate";
import i18n from "@/libs/i18n";
import middleware from "./middleware";
import pages from "./routes/pages/index";
import store from "@/store";

Vue.use(VueRouter);

const router = new VueRouter({
    mode: "history",
    base: process.env.BASE_URL,
    scrollBehavior(to, from, savedPosition) {
        if (to.hash) {
            return {
                selector: to.hash,
                behavior: "smooth",
            };
        }
        return { x: 0, y: 0 };
    },
    routes: [
        {
            path: "/",
            redirect: `/:lang`,
        },
        {
            path: "/:lang",
            component: {
                render(c) {
                    return c("router-view");
                },
            },
            children: [
                {
                    path: "",
                    name: "home",
                    component: () => import("@/views/pages/website/Index.vue"),
                    meta: {
                        middleware,
                        layout: "full",
                    },
                },
                {
                    path: "start-now",
                    name: "start-now",
                    component: () => import("@/views/pages/website/StartNow.vue"),
                    meta: {
                        middleware,
                        layout: "full",
                    },
                },
                {
                    path: "policy",
                    component: {
                        render(c) {
                            return c("router-view");
                        },
                    },
                    children: [
                        {
                            path: "",
                            name: "policy",
                            component: () =>
                                import("@/views/pages/website/Policy.vue"),
                            meta: {
                                middleware,
                                layout: "full",
                            },
                        },
                        {
                            path: "agencies",
                            name: "policy-agencies",
                            component: () =>
                                import(
                                    "@/views/pages/website/policy/Agencies.vue"
                                ),
                            meta: {
                                middleware,
                                layout: "full",
                            },
                        },
                        {
                            path: "companies",
                            name: "policy-companies",
                            component: () =>
                                import(
                                    "@/views/pages/website/policy/Companies.vue"
                                ),
                            meta: {
                                middleware,
                                layout: "full",
                            },
                        },
                    ],
                },
                {
                    path: "terms-and-conditions",
                    component: {
                        render(c) {
                            return c("router-view");
                        },
                    },
                    children: [
                        {
                            path: "",
                            name: "terms-and-conditions",
                            component: () =>
                                import(
                                    "@/views/pages/website/TermsAndConditions.vue"
                                ),
                            meta: {
                                middleware,
                                layout: "full",
                            },
                        },
                        {
                            path: "agencies",
                            name: "terms-and-conditions-agencies",
                            component: () =>
                                import(
                                    "@/views/pages/website/terms-and-conditions/Agencies.vue"
                                ),
                            meta: {
                                middleware,
                                layout: "full",
                            },
                        },
                        {
                            path: "companies",
                            name: "terms-and-conditions-companies",
                            component: () =>
                                import(
                                    "@/views/pages/website/terms-and-conditions/Companies.vue"
                                ),
                            meta: {
                                middleware,
                                layout: "full",
                            },
                        },
                    ],
                },
                {
                    path: `not-found`,
                    name: "404",
                    component: () => import("@/views/pages/404.vue"),
                    meta: {
                        middleware,
                        layout: "full",
                    },
                },
                ...pages,
            ],
        },
        {
            path: "*",
            redirect: `/${i18n.locale}/not-found`,
        },
    ],
});

// runs the default `next()` callback but also triggers
// the subsequent Middleware function.
function nextFactory(context, middleware, index) {
    const subsequentMiddleware = middleware[index];
    // If no subsequent Middleware exists,
    // the default `next()` callback is returned.
    if (!subsequentMiddleware) return context.next;

    return (...parameters) => {
        // Run the default Vue Router `next()` callback first.
        context.next(...parameters);
        // Then run the subsequent Middleware with a new
        // `nextMiddleware()` callback.
        const nextMiddleware = nextFactory(context, middleware, index + 1);
        subsequentMiddleware({ ...context, next: nextMiddleware });
    };
}

router.beforeEach((to, from, next) => {
    const langs = ["ar", "en"];
    const roles = ["business", "employee"];
    const loginData = Vue.$cookies.get("login-data", { parseJSON: true });
    if (langs.includes(to.params.lang)) {
        i18n.locale = to.params.lang;
        localize(to.params.lang);
        axios.defaults.headers.common.locale = to.params.lang;
        store.commit("TOGGLE_RTL", to.params.lang);
        if (to.meta.middleware) {
            const redirectedFrom = to.redirectedFrom
                ? to.redirectedFrom.split("/")[2]
                : null;
            if (roles.includes(redirectedFrom) && !loginData) {
                return next({
                    name: "login",
                    params: {
                        lang: i18n.locale,
                    },
                });
            }
            if (to.params.role && !roles.includes(to.params.role)) {
                return next({
                    name: "404",
                    params: {
                        lang: i18n.locale,
                    },
                });
            }

            if (to.params.role && roles.includes(to.params.role) && !to.name) {
                return next({
                    name: "dashboard",
                    params: { lang: i18n.locale, role: loginData.account_type },
                });
            }

            const middleware = Array.isArray(to.meta.middleware)
                ? to.meta.middleware
                : [to.meta.middleware];

            const context = {
                from,
                next,
                router,
                to,
            };
            const nextMiddleware = nextFactory(context, middleware, 1);
            return middleware[0]({ ...context, next: nextMiddleware });
        }
        if (loginData) {
            return next({
                name: "dashboard",
                params: {
                    lang: i18n.locale,
                    role: loginData.account_type,
                },
            });
        }
        return next({
            name: "login",
            params: {
                lang: i18n.locale,
            },
        });
    }

    if (to.name) {
        return next({ name: to.name, params: { lang: i18n.locale } });
    }
    return next({ name: "home", params: { lang: "ar" } });
});

// ? For splash screen
// Remove afterEach hook if you are not using splash screen
router.afterEach(() => {
    // Remove initial loading
    const appLoading = document.getElementById("loading-bg");
    if (appLoading) {
        appLoading.style.display = "none";
    }
});

export default router;
